/*
 * "Software pw3270, desenvolvido com base nos códigos fontes do WC3270  e X3270
 * (Paul Mattes Paul.Mattes@usa.net), de emulação de terminal 3270 para acesso a
 * aplicativos mainframe. Registro no INPI sob o nome G3270.
 *
 * Copyright (C) <2008> <Banco do Brasil S.A.>
 *
 * Este programa é software livre. Você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da GPL v.2 - Licença Pública Geral  GNU,  conforme  publicado  pela
 * Free Software Foundation.
 *
 * Este programa é distribuído na expectativa de  ser  útil,  mas  SEM  QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou  de  ADEQUAÇÃO
 * A QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 51 Franklin
 * St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Este programa está nomeado como - e possui - linhas de código.
 *
 * Referências:
 *
 * https://github.com/joprietoe/gdbus/blob/master/gdbus-example-server.c
 * https://github.com/bratsche/glib/blob/master/gio/tests/gdbus-example-export.c
 *
 * Contatos:
 *
 * perry.werneck@gmail.com	(Alexandre Perry de Souza Werneck)
 * erico.mendonca@gmail.com	(Erico Mascarenhas Mendonça)
 *
 */

#include <internals.h>

#include <glib.h>
#include <gio/gio.h>

#include <ipc-glib.h>
#include <lib3270.h>

#include <lib3270/actions.h>
#include <lib3270/properties.h>
#include <lib3270/toggle.h>
#include <v3270.h>

#ifndef _WIN32
	#include <dbus/dbus-glib.h>
	#include <dbus/dbus-glib-bindings.h>
#endif // _WIN32

/*--[ Widget definition ]----------------------------------------------------------------------------*/

struct _ipc3270Response {
	GObject		  parent;
	GList		* values;
};

struct _ipc3270ResponseClass {

	GObjectClass parent;
	int dummy;

};


G_DEFINE_TYPE(ipc3270Response, ipc3270Response, G_TYPE_OBJECT)

/*--[ Implement ]------------------------------------------------------------------------------------*/

static void ipc3270Response_finalize(GObject *object) {

	ipc3270Response * response = IPC3270_RESPONSE(object);

	if(response->values)
		g_list_free_full(response->values,(GDestroyNotify) g_variant_unref);

}


static void ipc3270Response_class_init(ipc3270ResponseClass *klass) {

	debug("%s",__FUNCTION__);

	GObjectClass *object_class;
	object_class = G_OBJECT_CLASS (klass);
	object_class->finalize = ipc3270Response_finalize;

}

static void ipc3270Response_init(ipc3270Response *object) {

	object->values = NULL;

}

GObject * ipc3270_response_new() {
	return g_object_new(GLIB_TYPE_IPC3270_RESPONSE, NULL);
}

void ipc3270_response_append(GObject *object, GVariant *value) {
	ipc3270Response * response = IPC3270_RESPONSE(object);
	response->values = g_list_append(response->values,value);
}

void ipc3270_response_append_int32(GObject *object, gint32 value) {
	ipc3270_response_append(object,g_variant_new_int32(value));
}

void ipc3270_response_append_uint32(GObject *object, guint32 value) {
	ipc3270_response_append(object,g_variant_new_uint32(value));
}

void ipc3270_response_append_string(GObject *object, const gchar *text) {
	ipc3270_response_append(object,g_variant_new_string(text));
}

void ipc3270_response_append_boolean(GObject *object, gboolean value) {
	ipc3270_response_append(object,g_variant_new_boolean(value));
}

GVariant * ipc3270_response_steal_value(GObject *object) {

	ipc3270Response * response = IPC3270_RESPONSE(object);

	GList *first = g_list_first(response->values);
	GVariant * value = first->data;

	response->values = g_list_remove(response->values,value);

	return value;
}

const GList	* ipc3270_get_values(GObject *object) {
	return IPC3270_RESPONSE(object)->values;
}

guint ipc3270_response_length(GObject *object) {
	return g_list_length(IPC3270_RESPONSE(object)->values);
}

gboolean ipc3270_response_has_values(GObject *object) {
	return IPC3270_RESPONSE(object)->values != NULL;
}
